@@ -1,10 +1,14 @@
+"""Hyper transformer module."""
+
 import re
+import warnings
+from copy import deepcopy
 
 import numpy as np
 
 from rdt.transformers import (
-    BooleanTransformer, CategoricalTransformer, DatetimeTransformer, NumericalTransformer,
-    load_transformers)
+    BooleanTransformer, CategoricalTransformer, DatetimeTransformer, LabelEncodingTransformer,
+    NumericalTransformer, OneHotEncodingTransformer, load_transformers)
 
 
 class HyperTransformer:
@@ -26,6 +30,10 @@ class HyperTransformer:
         dtypes (list or None):
             List of column data types to use when building the ``transformers`` dict
             automatically. If not passed, the ``DataFrame.dtypes`` are used.
+        dtype_transformers (dict or None):
+            Transformer templates to use for each dtype. Passed as a dictionary of
+            dtype kinds ('i', 'f', 'O', 'b', 'M') and transformer names, classes
+            or instances.
 
     Example:
         Create a simple ``HyperTransformer`` instance that will decide which transformers
@@ -50,12 +58,36 @@ class HyperTransformer:
         ... }
         >>> ht = HyperTransformer(transformers)
     """
-    def __init__(self, transformers=None, copy=True, anonymize=None, dtypes=None):
+
+    _TRANSFORMER_TEMPLATES = {
+        'numerical': NumericalTransformer,
+        'integer': NumericalTransformer(dtype=int),
+        'float': NumericalTransformer(dtype=float),
+        'categorical': CategoricalTransformer,
+        'categorical_fuzzy': CategoricalTransformer(fuzzy=True),
+        'one_hot_encoding': OneHotEncodingTransformer(error_on_unknown=False),
+        'label_encoding': LabelEncodingTransformer,
+        'boolean': BooleanTransformer,
+        'datetime': DatetimeTransformer,
+    }
+    _DTYPE_TRANSFORMERS = {
+        'i': 'numerical',
+        'f': 'numerical',
+        'O': 'categorical',
+        'b': 'boolean',
+        'M': 'datetime',
+    }
+
+    def __init__(self, transformers=None, copy=True, anonymize=None,
+                 dtypes=None, dtype_transformers=None):
         self.transformers = transformers
         self._transformers = dict()
         self.copy = copy
         self.anonymize = anonymize or dict()
         self.dtypes = dtypes
+        self.dtype_transformers = self._DTYPE_TRANSFORMERS.copy()
+        if dtype_transformers:
+            self.dtype_transformers.update(dtype_transformers)
 
     def _analyze(self, data):
         """Build a ``dict`` with column names and transformers from a given ``pandas.DataFrame``.
@@ -84,31 +116,39 @@ class HyperTransformer:
                 if a ``dtype`` is not supported by the `HyperTransformer``.
         """
         transformers = dict()
-        dtypes = self.dtypes or data.dtypes
         if self.dtypes:
             dtypes = self.dtypes
         else:
             dtypes = [
-                data[column].dropna().infer_objects()
+                data[column].dropna().infer_objects().dtype
                 for column in data.columns
             ]
 
         for name, dtype in zip(data.columns, dtypes):
-            dtype = np.dtype(dtype)
-            if dtype.kind == 'i':
-                transformer = NumericalTransformer(dtype=int)
-            elif dtype.kind == 'f':
-                transformer = NumericalTransformer(dtype=float)
-            elif dtype.kind == 'O':
-                anonymize = self.anonymize.get(name)
-                transformer = CategoricalTransformer(anonymize=anonymize)
-            elif dtype.kind == 'b':
-                transformer = BooleanTransformer()
-            elif dtype.kind == 'M':
-                transformer = DatetimeTransformer()
-            else:
+            try:
+                kind = np.dtype(dtype).kind
+            except TypeError:
+                # probably category
+                kind = 'O'
+
+            transformer_template = self.dtype_transformers[kind]
+            if not transformer_template:
                 raise ValueError('Unsupported dtype: {}'.format(dtype))
 
+            if isinstance(transformer_template, str):
+                transformer_template = self._TRANSFORMER_TEMPLATES[transformer_template]
+
+            if not isinstance(transformer_template, type):
+                transformer = deepcopy(transformer_template)
+            elif self.anonymize and transformer_template == CategoricalTransformer:
+                warnings.warn(
+                    'Categorical anonymization is deprecated and will be removed from RDT soon.',
+                    DeprecationWarning
+                )
+                transformer = CategoricalTransformer(anonymize=self.anonymize)
+            else:
+                transformer = transformer_template()
+
             transformers[name] = transformer
 
         return transformers
@@ -151,10 +191,10 @@ class HyperTransformer:
 
             shape = transformed.shape
 
-            if len(shape) == 2 and shape[1] == 2:
-                data[column_name] = transformed[:, 0]
-                new_column = '{}#{}'.format(column_name, 1)
-                data[new_column] = transformed[:, 1]
+            if len(shape) == 2:
+                for index in range(shape[1]):
+                    new_column = '{}#{}'.format(column_name, index)
+                    data[new_column] = transformed[:, index]
 
             else:
                 data[column_name] = transformed
